///////////////////////////////////////////////////////////////////////////
//                                                                       //
//            File: modem.cpp                                            //
//                  (project version).                                   //
//            started on: 24/2/92                                        //
//                                                                       //
///////////////////////////////////////////////////////////////////////////
//                                                                       //
//                                                                       //                                                                     //
///////////////////////////////////////////////////////////////////////////

#include <dos.h>    // inportb, delay.
#include "modem.h"  // MODEM

const char modem_disconnect_string[]= "ATH\n";
const char modem_connect_string[]= "AT&F2\n";
const char modem_escape_sequence[]= "+++";

// extending the init.
//
void MODEM::extended_init(void)
{
    const char* connect_string= modem_connect_string;

    // turn on DTR, RTS.
    //
    set_controls(0x3);

    for (int i=0; connect_string[i]!='\x0'; i++) {
        *this<< connect_string[i];
    }
}

// destructor.
//
MODEM::~MODEM()
{
    // turn off DTR, RTS.
    //
    set_controls(0);
}

void MODEM::flow_check(void)
{
    // check flow control.
    // if both DSR && CTS are activated:- flow is enabled.
    //
    flow_enabled= ((get_MSR()& 0x30)== 0x30)?TRUE: FALSE;
}

void MODEM::disconnect(void)
{
    const char* disconnect_string= modem_disconnect_string;
    const char* escape_sequence= modem_escape_sequence;
    static int i;

    if (!(get_MSR()& 0x8))
       return; // if DCD is down, no need...

    set_controls(0);
    delay (1000);
    set_controls(0x3);

    for (i=0; escape_sequence[i]!='\x0'; i++) {
        *this<< escape_sequence[i];
    }

    delay (1000);

    for (i=0; disconnect_string[i]!='\x0'; i++) {
        *this<< disconnect_string[i];
    }
    // if DCD is up now, line was not disconnected!!!

//    if (get_MSR()& 0x8)
//       return; // if DCD is down, no need...

}

void MODEM::msr_int(const BYTE msr)
{
    if ((msr& 0x44)== 0x44) // ring is up and the interrupt is because of it.
       ring_int (msr);

    if (msr& 0x3)   // DSR or CTS changed!
       modem_status_change (msr);

    if ((msr& 0x8)) // DCD changed.
       dcd_status_change (msr);
}

void MODEM::dcd_status_change(BYTE /* msr */)
{
    // if DCD or CTS changed...

    // to be overloaded.
}

void MODEM::modem_status_change(BYTE /* msr */)
{
    // if DSR or CTS changed...

    // to be overloaded.
}

void MODEM::ring_int(BYTE /* msr */)
{
    // to be overloaded.
}

BYTE MODEM::get_MSR(void)
{
    return ((BYTE) inportb(com_port+ MSR));
}
